@@ -45,7 +45,15 @@
 #include "config.h"
 #endif
 
+/* Jong@09022009 */
+#ifdef XORG_VERSION_CURRENT
+#include "xorgVersion.h"
+
+#if (XORG_VERSION_CURRENT >= XORG_VERSION_NUMERIC(6,9,0,0,0) )
 #define VC //video capture
+#endif
+
+#endif
 
 # ifdef VC
 #include <stdio.h>
@@ -465,8 +473,12 @@ XGISetupImageVideo(ScreenPtr pScreen)
 # endif //VC
 
     pPriv->currentBuf = 0;
+
+#ifdef XGI_USE_XAA
     pPriv->linear     = NULL;
     pPriv->fbAreaPtr = NULL;
+#endif
+
     pPriv->fbSize = 0;
 	pPriv->videoStatus = 0;
 	pPriv->linebufMergeLimit = 1280;
@@ -1039,12 +1051,13 @@ XGIStopVideo(ScrnInfoPtr pScrn, pointer data, Bool exit)
      }
 # endif //VC	
 	
+#ifdef XGI_USE_XAA
      if(pPriv->fbAreaPtr) {
        xf86FreeOffscreenArea(pPriv->fbAreaPtr);
        pPriv->fbAreaPtr = NULL;
        pPriv->fbSize = 0;
      }
-	
+#endif	
 	
      /* clear all flag */
      pPriv->videoStatus = 0;
@@ -1145,14 +1158,17 @@ XGIPutImage(
       
       pPriv->fbSize = totalSize;
 
+#ifdef XGI_USE_XAA
       if(pPriv->fbAreaPtr) {
                 xf86FreeOffscreenArea(pPriv->fbAreaPtr);
         }
-      
+#endif
+
       depth = (pScrn->bitsPerPixel + 7 ) / 8;
       pitch = pScrn->displayWidth * depth;
       lines = ((totalSize * 2) / pitch) + 1;
               
+#ifdef XGI_USE_XAA
       pPriv->fbAreaPtr = xf86AllocateOffscreenArea(pScrn->pScreen, 
                                  pScrn->displayWidth,
                                 lines, 0, NULL, NULL, NULL);
@@ -1166,7 +1182,7 @@ XGIPutImage(
       pBox = &(pPriv->fbAreaPtr->box);     
       pPriv->bufAddr[0] = (pBox->x1 * depth) + (pBox->y1 * pitch); 
       pPriv->bufAddr[1] = pPriv->bufAddr[0] + totalSize;
-      
+#endif      
    } while(0);
 	
    /* copy data */
